const uuid = require('uuid')

const getInvitationData = (invitations, userId, role) => {
  const matchingInvitation = invitations.find(
    inv => inv.role === role && inv.userId === userId,
  )
  let status = 'pending'
  if (matchingInvitation.isAccepted) {
    status = 'accepted'
  } else if (matchingInvitation.hasAnswer) {
    status = 'refused'
  }

  const { timestamp, id } = matchingInvitation
  return { timestamp, status, id }
}

const setupInvitation = async (userId, role, collection) => {
  const invitation = {
    role,
    hasAnswer: false,
    isAccepted: false,
    timestamp: Date.now(),
    id: uuid.v4(),
    userId,
  }
  collection.invitations = collection.invitations || []
  collection.invitations.push(invitation)
  collection = await collection.save()
  return invitation
}

const setupReviewerInvitation = async (
  req,
  models,
  collection,
  user,
  mailService,
) => {
  const baseUrl = `${req.protocol}://${req.get('host')}`
  const fragment = await models.Fragment.find(collection.fragments[0])
  const submittingAuthorData = collection.authors.find(
    author => author.isSubmitting === true,
  )
  const submittingAuthor = await models.User.find(submittingAuthorData.userId)
  const authorsPromises = collection.authors.map(async author => {
    const user = await models.User.find(author.userId)
    return `${user.firstName} ${user.lastName}`
  })
  const authors = await Promise.all(authorsPromises)
  let { abstract, title } = fragment.metadata
  title = title.replace(/<(.|\n)*?>/g, '')
  abstract = abstract ? abstract.replace(/<(.|\n)*?>/g, '') : ''
  await mailService.setupReviewerInvitationEmail(
    user,
    baseUrl,
    collection,
    abstract,
    title,
    submittingAuthor,
    authors,
  )
}

module.exports = {
  getInvitationData,
  setupInvitation,
  setupReviewerInvitation,
}
