import React from 'react'
import { connect } from 'react-redux'
import { Button, th } from '@pubsweet/ui'
import styled, { css } from 'styled-components'
import { compose, withHandlers } from 'recompose'
import {
  withModal,
  ConfirmationModal,
} from 'pubsweet-component-modal/src/components'
import {
  selectFetchingDecision,
  reviewerAccept,
  reviewerDecline,
} from '../../redux/reviewers'

const ReviewerDecision = ({ showAcceptModal, showDeclineModal }) => (
  <div>
    <DecisionButton onClick={showDeclineModal}>Decline</DecisionButton>
    <DecisionButton onClick={showAcceptModal} primary>
      Accept
    </DecisionButton>
  </div>
)

const ModalComponent = connect(state => ({
  isFetching: selectFetchingDecision(state),
}))(({ isFetching, ...rest }) => (
  <ConfirmationModal {...rest} isFetching={isFetching} />
))

export default compose(
  connect(null, {
    reviewerAccept,
    reviewerDecline,
  }),
  withModal({
    modalKey: 'reviewer-decision',
    modalComponent: ModalComponent,
  }),
  withHandlers({
    showAcceptModal: ({ showModal, hideModal, reviewerAccept }) => () => {
      showModal({
        title: 'Agree to review Manuscript?',
        onConfirm: () => {
          reviewerAccept().then(hideModal)
        },
      })
    },
    showDeclineModal: ({ showModal, hideModal, reviewerDecline }) => () => {
      showModal({
        title: 'Decline to review Manuscript?',
        onConfirm: () => {
          reviewerDecline().then(hideModal)
        },
      })
    },
  }),
)(ReviewerDecision)

// #region styled-components
const defaultText = css`
  font-family: ${th('fontReading')};
  font-size: ${th('fontSizeBaseSmall')};
`

const DecisionButton = styled(Button)`
  ${defaultText};
  align-items: center;
  color: ${({ primary }) =>
    primary ? th('colorTextReverse') : th('colorPrimary')});
  background-color: ${({ primary }) =>
    primary ? th('colorPrimary') : th('backgroundColorReverse')};
  height: calc(${th('subGridUnit')}*5);
  text-align: center;
`
// #endregion
