import React from 'react'
import { connect } from 'react-redux'
import { reduxForm } from 'redux-form'
import { required, minChars } from 'xpub-validators'
import { Button, ValidatedField, TextField } from '@pubsweet/ui'
import { compose, withState, lifecycle } from 'recompose'

import {
  Row,
  Err,
  Title,
  Label,
  Email,
  RowItem,
  Subtitle,
  RootContainer,
  FormContainer,
} from './FormItems'
import { parseSearchParams } from './utils'
import {
  setReviewerPassword,
  reviewerAccept,
  reviewerDecline,
} from '../../redux/reviewers'

const min8Chars = minChars(8)
const ReviewerSignUp = ({ handleSubmit, error, reviewer }) => (
  <RootContainer>
    <Title>Hindawi Invitation</Title>
    <Subtitle>
      You have been invited to review a manuscript on the Hindawi platform.
      Please set a password and proceed to the manuscript.
    </Subtitle>
    <Email>{reviewer.email}</Email>
    <FormContainer onSubmit={handleSubmit}>
      <Row>
        <RowItem>
          <Label> Password </Label>
          <ValidatedField
            component={input => <TextField {...input} type="password" />}
            name="password"
            validate={[required, min8Chars]}
          />
        </RowItem>
      </Row>
      {error && (
        <Row>
          <RowItem>
            <Err>{error}</Err>
          </RowItem>
        </Row>
      )}
      <Row>
        <Button primary type="submit">
          CONFIRM
        </Button>
      </Row>
    </FormContainer>
  </RootContainer>
)

export default compose(
  withState('reviewer', 'setReviewer', {}),
  connect(null, { setReviewerPassword, reviewerAccept, reviewerDecline }),
  lifecycle({
    componentDidMount() {
      const { location, setReviewer, reviewerDecline } = this.props
      const { decision = true, ...reviewerBody } = parseSearchParams(
        location.search,
      )

      if (decision) {
        setReviewer(reviewerBody)
      } else {
        reviewerDecline()
      }
    },
  }),
  reduxForm({
    form: 'invite-reviewer',
    onSubmit: (
      { password },
      dispatch,
      { reviewer, location, setReviewerPassword },
    ) => {
      setReviewerPassword({
        ...reviewer,
        password,
      })
    },
  }),
)(ReviewerSignUp)
